/*
   Various Color functions

   Concept, Design and Implementation: Craig A. Lindley

   Last Update: 08/28/2020
*/
#ifndef COLORS_H
#define COLORS_H

// Input a value 0 to 255 to get a color value.
// The colours are a transition r - g - b - back to r.
uint32_t Wheel(byte pos) {
  int red, grn, blu;

  pos = 255 - pos;

  if (pos < 85) {
    red = 255 - pos * 3;
    grn = 0;
    blu = pos * 3;
  } else if (pos < 170) {
    pos -= 85;
    red = 0;
    grn = pos * 3;
    blu = 255 - pos * 3;
  } else {
    pos -= 170;
    red = pos * 3;
    grn = 255 - pos * 3;
    blu = 0;
  }
  return Adafruit_NeoPixel::Color(red, grn, blu);
}

uint32_t randomColor() {
  return Wheel(random8(255));
}

// HSV to RGB color conversion
// Input arguments
// hue in degrees (0 - 360.0)
// saturation (0.0 - 1.0)
// value (0.0 - 1.0)
// Output arguments
// red, green blue (0.0 - 1.0)
void hsvToRGB(float hue, float saturation, float value, float *red, float *green, float *blue) {

  int i;
  float f, p, q, t;

  if (saturation == 0) {
    // achromatic (grey)
    *red = *green = *blue = value;
    return;
  }
  hue /= 60;                  // sector 0 to 5
  i = floor(hue);
  f = hue - i;                // factorial part of h
  p = value * (1 - saturation);
  q = value * (1 - saturation * f);
  t = value * (1 - saturation * (1 - f));
  switch (i) {
    case 0:
      *red = value;
      *green = t;
      *blue = p;
      break;
    case 1:
      *red = q;
      *green = value;
      *blue = p;
      break;
    case 2:
      *red = p;
      *green = value;
      *blue = t;
      break;
    case 3:
      *red = p;
      *green = q;
      *blue = value;
      break;
    case 4:
      *red = t;
      *green = p;
      *blue = value;
      break;
    default:
      *red = value;
      *green = p;
      *blue = q;
      break;
  }
}

uint32_t getHSVColor(float hue, float saturation, float value) {
  float red, green, blue;

  // Values return are floats from 0.0 .. 1.0
  hsvToRGB(hue, saturation, value, &red, &green, &blue);

  return Adafruit_NeoPixel::Color(red * 255, green * 255, blue * 255);
}

// Return a red color with 100% saturation and variable value.
// Low values of value trend towards black
uint32 getShadeOfRed(int r) {

  int red = map(r, 0, 7, 40, 8);

  return Adafruit_NeoPixel::Color(red, 0, 0);
}

#endif
