/*
   More New Pattern Functions

   Concept, Design and Implementation: Craig A. Lindley
   Last Update: 08/30/2020
*/

#ifndef NEW_PATTERNS_2_H
#define NEW_PATTERNS_2_H

/***********************************************************************/
/*                              Lava Pattern                           */
/***********************************************************************/

typedef struct {
  int x;
  int y;
} POINT;

// Paths of lava coming out of the middle on top panel
// and ending on the perimeter of the panel. There are
// a total of 28 unique paths numbered 0 .. 27. Each path
// consists of four pixels which are drawn sequentially.
POINT path0[]  = {3, 3, 2, 2, 1, 1, 0, 0};
POINT path1[]  = {3, 3, 2, 2, 2, 1, 1, 0};
POINT path2[]  = {3, 3, 3, 2, 2, 1, 2, 0};
POINT path3[]  = {3, 3, 3, 2, 3, 1, 3, 0};
POINT path4[]  = {4, 3, 4, 2, 4, 1, 4, 0};
POINT path5[]  = {4, 3, 4, 2, 5, 1, 5, 0};
POINT path6[]  = {4, 3, 5, 2, 5, 1, 6, 0};
POINT path7[]  = {4, 3, 5, 2, 6, 1, 7, 0};
POINT path8[]  = {4, 3, 5, 2, 6, 2, 7, 1};
POINT path9[]  = {4, 3, 5, 3, 6, 2, 7, 2};
POINT path10[] = {4, 3, 5, 3, 6, 3, 7, 3};
POINT path11[] = {4, 4, 5, 4, 6, 4, 7, 4};
POINT path12[] = {4, 4, 5, 4, 6, 5, 7, 5};
POINT path13[] = {4, 4, 5, 5, 6, 5, 7, 6};
POINT path14[] = {4, 4, 5, 5, 6, 6, 7, 7};
POINT path15[] = {4, 4, 5, 5, 5, 6, 6, 7};
POINT path16[] = {4, 4, 4, 5, 5, 6, 5, 7};
POINT path17[] = {4, 4, 4, 5, 4, 6, 4, 7};
POINT path18[] = {3, 4, 3, 5, 3, 6, 3, 7};
POINT path19[] = {3, 4, 3, 5, 2, 6, 2, 7};
POINT path20[] = {3, 4, 2, 5, 2, 6, 1, 7};
POINT path21[] = {3, 4, 2, 5, 1, 6, 0, 7};
POINT path22[] = {3, 4, 2, 5, 1, 5, 0, 6};
POINT path23[] = {3, 4, 2, 4, 1, 5, 0, 5};
POINT path24[] = {3, 4, 2, 4, 1, 4, 0, 4};
POINT path25[] = {3, 3, 2, 3, 1, 3, 0, 3};
POINT path26[] = {3, 3, 2, 3, 1, 2, 0, 2};
POINT path27[] = {3, 3, 2, 2, 1, 2, 0, 1};

// Array of pointers to the paths
POINT * paths[] = {
  path0, path1, path2, path3, path4, path5, path6,
  path7, path8, path9, path10, path11, path12, path13,
  path14, path15, path16, path17, path18, path19, path20,
  path21, path22, path23, path24, path25, path26, path27
};

// Shade of white
uint32_t WHITE_1 = Adafruit_NeoPixel::Color(200, 200, 200);
uint32_t WHITE_2 = Adafruit_NeoPixel::Color(150, 140, 130);
uint32_t WHITE_3 = Adafruit_NeoPixel::Color(90, 90, 90);
uint32_t WHITE_4 = Adafruit_NeoPixel::Color(50, 50, 50);

// Pixels that make up center of top panel where crater would be
int centerP1Offset = 27;
int centerP2Offset = 28;
int centerP3Offset = 35;
int centerP4Offset = 36;

// Flash the neopixels N times
void flashTopCenter(int times) {
  yield();
  for (int t = 0; t < times; t++) {
    // Turn center neopixels on
    matrix.setPixelColor(centerP1Offset, WHITE_1);
    matrix.show();
    delay(random8(30));
    matrix.setPixelColor(centerP2Offset, WHITE_2);
    matrix.show();
    delay(random8(30));
    matrix.setPixelColor(centerP3Offset, WHITE_3);
    matrix.show();
    delay(random8(30));
    matrix.setPixelColor(centerP4Offset, WHITE_4);
    matrix.show();
    delay(random8(50));
    matrix.setPixelColor(centerP2Offset, 0);
    matrix.show();
    delay(random8(30));
    matrix.setPixelColor(centerP1Offset, 0);
    matrix.show();
    delay(random8(30));
    matrix.setPixelColor(centerP4Offset, 0);
    matrix.show();
    delay(random8(30));
    matrix.setPixelColor(centerP3Offset, 0);
    matrix.show();
    delay(random8(50));
  }
}

// Propagate the lava (LEDs) down the sides of the cube dulling the lava
// as it falls
void propagateLava() {

  uint32_t faceBuffer[8][8];
  uint32_t color;
  int faceOffset, pixelOffset;
  int newCol;

  // Do 8 steps of lava movement towards bottom of panels
  for (int s = 0; s < 8; s++) {

    yield();

    // Propagate all four faces: left, front, right and back
    for (int f = 0; f < 4; f++) {

      yield();

      // Determine offset to neopixel panel
      switch (f) {
        case 0:
          faceOffset = FACE_LEFT_OFFSET;
          break;
        case 1:
          faceOffset = FACE_FRONT_OFFSET;
          break;
        case 2:
          faceOffset = FACE_RIGHT_OFFSET;
          break;
        case 3:
          faceOffset = FACE_BACK_OFFSET;
          break;
      }

      // Clear face buffer
      memset(faceBuffer, 0, sizeof(faceBuffer));

      // Lava leds only flow down a max of eight times
      for (int row = s; row < 8; row++) {

        yield();

        for (int col = 0; col < 8; col++) {
          pixelOffset = faceOffset + row * 8 + col;
          color = matrix.getPixelColor(pixelOffset);

          // Does this pixel have color?
          if (color != 0) {
            // Yes it does so process it. Processing consists of
            // dulling its color and possibly altering its track
            newCol = col;

            if (random8(4) == 3) {
              // There is a 1 in 4 chance of a course change
              switch (random8(3)) {
                case 0:
                  // Course moves to the left if possible
                  newCol = col - 1;
                  if (newCol < 0) {
                    newCol = 1;
                  }
                  break;
                case 1:
                  // No course change
                  break;
                case 2:
                  // Course moves to the right if possible
                  newCol = col + 1;
                  if (newCol > 7) {
                    newCol = 6;
                  }
                  break;
              }
            }
            // There is also a 1 in 20 chance of lava cooling to black
            // before it reaches the bottom
            if (random8(20) == 5) {
              color = 0;
            }
            if (row < 7) {
              // Log the new position and indicate it has color
              faceBuffer[row + 1][newCol] = color;
            }
          }
        }
      }
      // Move pixel data from faceBuffer to neopixel panel
      // Lava gets cooler and duller as it flows down hill
      // Actual color of the lava is assigned here
      for (int row = 0; row < 8; row++) {
        yield();
        for (int col = 0; col < 8; col++) {
          color = faceBuffer[row][col];
          if (color != 0) {
            pixelOffset = faceOffset + row * 8 + col;
            matrix.setPixelColor(pixelOffset, getShadeOfRed(row));
          }
        }
      }
      matrix.show();
      delay(70);
    }
  }
}

// Max number of lava streams leaving top of cube
#define MAX_STREAMS 20

// Volcano/Lava simulation
void lava(int loops) {

  int streamLines[MAX_STREAMS];

  for (int l = 0; l < loops; l++) {

    yield();

    // Clear cube each iteration
    matrix.clear();

    // Flash the white hot lava in the crater
    flashTopCenter(5);

    // Randomly determine how many streams of lava to display
    int streams = random8(5, MAX_STREAMS);
    for (int i = 0; i < streams; i++) {
      streamLines[i] = random8(28);
    }

    // Light the four LEDs in each stream line/path
    for (int led = 0; led < 4; led++) {
      // Do this for each stream
      for (int stream = 0; stream < streams; stream++) {
        yield();
        POINT *path = paths[streamLines[stream]];
        POINT pt = path[led];
        matrix.setPixelColor(pt.y * 8 + pt.x, Adafruit_NeoPixel::Color(255, 0, 0));
      }
      matrix.show();
      delay(400);
    }
    // Now set a pixel on each cube face where the lava will propagate from
    for (int stream = 0; stream < streams; stream++) {
      yield();
      int endPoint = streamLines[stream];
      if ((endPoint >= 0) && (endPoint < 8)) {
        matrix.setPixelColor(FACE_BACK_OFFSET + 7 - endPoint, Adafruit_NeoPixel::Color(255, 0, 0));
      } else if ((endPoint >= 8) && (endPoint < 15)) {
        matrix.setPixelColor(FACE_RIGHT_OFFSET + 14 - endPoint, Adafruit_NeoPixel::Color(255, 0, 0));
      } else if ((endPoint >= 15) && (endPoint < 22)) {
        matrix.setPixelColor(FACE_FRONT_OFFSET + 21 - endPoint, Adafruit_NeoPixel::Color(255, 0, 0));
      } else {
        matrix.setPixelColor(FACE_LEFT_OFFSET + 28 - endPoint, Adafruit_NeoPixel::Color(255, 0, 0));
      }
    }
    matrix.show();

    // Flash the white hot lava in the crater
    flashTopCenter(3);

    // Propagate the lava down the sides of the cube
    propagateLava();
  }
}

#endif
