/*
   Open Weather Map Code
   retrieves the current weather conditions

   Written by: Craig A. Lindley
   Last Update: 09/09/2020
*/

#include <JsonListener.h>
#include <JsonStreamingParser.h>

#include <ESP8266HTTPClient.h>

// Weather data structure
typedef struct OWMCurrentData {
  char main[40];
  float temp;
  float feelsLikeTemp;
  float windSpeed;
  uint8_t humidity;

} OWMCurrentData;

class OWMCurrentWeather: public JsonListener {

  public:
    OWMCurrentWeather(char *appId, char *zip) {

      // Build URL string in buffer using zip and app ID
      // This string is static and will not change so it only
      // needs to be generated once.
      sprintf(buffer,
              "http://api.openweathermap.org/data/2.5/weather?zip=%s&appid=%s&units=imperial&lang=en", zip, appId);
    }

    void getCurrentWeatherData(OWMCurrentData *data) {
      doUpdate(data);
    }

    //    String getMeteoconIcon(String icon);

    void whitespace(char c) {
      //        Serial.println("whitespace");
    }

    void startDocument() {
      //      Serial.println("Start document");
    }

    void endDocument() {
      done = true;
    }

    void startArray() {
      //      Serial.println("Start array");
    }

    void endArray() {
      //      Serial.println("End array");;
    }

    void startObject() {
      currentParent = currentKey;
    }

    void endObject() {
      if (currentParent == "weather") {
        weatherItemCounter++;
      }
      currentParent = "";
    }

    void key(String key) {
      currentKey = key;
    }

    void value(String value) {

      // Get only the first item if more than one is available
      if (currentParent == "weather" && weatherItemCounter == 0) {
        if (currentKey == "main") {
          strcpy(this->data->main, value.c_str());
        }
      }

      if (currentKey == "temp") {
        this->data->temp = value.toFloat();
      }
      if (currentKey == "feels_like") {
        this->data->feelsLikeTemp = value.toFloat();
      }
      if (currentKey == "speed") {
        this->data->windSpeed = value.toFloat();
      }
      if (currentKey == "humidity") {
        this->data->humidity = value.toInt();
      }
    }

  private:
    char buffer[150];
    String currentKey;
    String currentParent;
    OWMCurrentData *data;
    boolean done;
    uint8_t weatherItemCounter = 0;


    void doUpdate(OWMCurrentData *data) {

      done = false;

      unsigned long lostTest = 10000UL;
      unsigned long lost_do = millis();
      this->weatherItemCounter = 0;
      this->data = data;
      JsonStreamingParser parser;
      parser.setListener(this);
      HTTPClient http;

      // Serial.println(buffer);

      http.begin(buffer);
      bool isBody = false;
      char c;
      int size;
      //      Serial.print("[HTTP] GET...\n");
      // start connection and send HTTP header
      int httpCode = http.GET();
      //      Serial.printf("[HTTP] GET... code: %d\n", httpCode);
      if (httpCode == 200) {

        WiFiClient * client = http.getStreamPtr();

        while ((client->connected() || client->available()) && (! done)) {
          while ((size = client->available()) > 0) {
            if ((millis() - lost_do) > lostTest) {
              //              Serial.println ("lost in client with a timeout");
              client->stop();
              ESP.restart();
            }
            c = client->read();
            if (c == '{' || c == '[') {

              isBody = true;
            }
            if (isBody) {
              parser.parse(c);
            }
            // give WiFi and TCP/IP libraries a chance to handle pending events
            yield();
          }
        }
      }
      this->data = nullptr;
    }
};
